@@ -26,6 +26,7 @@ extern "C" {
 
 void gui_app(void);
 void gui_render_panel(void);
+bool gui_pan_scroll_behavior(int dir);
 }
 
 #ifndef typeof
@@ -67,9 +68,14 @@ void gui_render_panel(void);
 // How much space we keep for the labels on the left.
 static const float LABEL_SIZE = 90;
 
-// Base height of items (note: maybe remove and use the font size instead?).
-static const float ITEM_HEIGHT = 18;
-static const float ICON_HEIGHT = 32;
+#ifndef GUI_ITEM_HEIGHT
+#   define GUI_ITEM_HEIGHT 18
+#endif
+
+#ifndef GUI_ICON_HEIGHT
+#   define GUI_ICON_HEIGHT 32
+#endif
+
 static const ImVec2 ITEM_SPACING = ImVec2(8, 4);
 
 #define COL_HEX(x) ImVec4( \
@@ -140,9 +146,9 @@ enum {
 };
 
 static const char *ATTR_NAMES[] = {
-    [A_POS_LOC]         = "a_pos",
-    [A_TEX_POS_LOC]     = "a_tex_pos",
-    [A_COLOR_LOC]       = "a_color",
+    "a_pos",
+    "a_tex_pos",
+    "a_color",
     NULL
 };
 
@@ -153,11 +159,22 @@ typedef struct gui_t {
     GLuint  array_buffer;
     GLuint  index_buffer;
     margins_t margins;
-    bool    is_scrolling;
+
+    // bitmask: 1 - some window is scrolling.
+    //          2 - some window was scrolling last frame.
+    int     scrolling;
+
+    int     can_move_window;
+    bool    want_capture_mouse;
 
     int     is_row;
     float   item_size;
 
+    bool    is_context_menu;
+    int     context_menu_row;
+
+    int     win_dir; // Store the current window direction (for scrolling).
+
     struct {
         const char *title;
         int       (*func)(void *data);
@@ -167,6 +184,17 @@ typedef struct gui_t {
         bool        opened;
     } popup[8]; // Stack of modal popups
     int popup_count;
+
+    bool item_activated;
+    bool item_deactivated;
+
+    // Stack of the groups.
+    int groups_count;
+    struct {
+        bool activated;
+        bool deactivated;
+    } groups[16];
+
 } gui_t;
 
 static gui_t *gui = NULL;
@@ -301,28 +329,52 @@ static void ImImpl_RenderDrawLists(ImDrawData* draw_data)
     GL(glDisable(GL_SCISSOR_TEST));
 }
 
-static void load_fonts_texture()
+static void add_font(const char *uri, const ImWchar *ranges, bool mergmode)
 {
     ImGuiIO& io = ImGui::GetIO();
-
     float scale = goxel.screen_scale;
-    unsigned char* pixels;
-    int width, height;
     const void *data;
     int data_size;
     ImFontConfig conf;
 
+    conf.FontDataOwnedByAtlas = false;
+    conf.MergeMode = mergmode;
+    data = assets_get(uri, &data_size);
+    assert(data);
+    io.Fonts->AddFontFromMemoryTTF(
+            (void*)data, data_size, 14 * scale, &conf, ranges);
+}
+
+static void load_fonts_texture()
+{
+    ImGuiIO& io = ImGui::GetIO();
+
+    unsigned char* pixels;
+    int width, height;
     const ImWchar ranges[] = {
         0x0020, 0x00FF, // Basic Latin + Latin Supplement
         0x25A0, 0x25FF, // Geometric shapes
         0
     };
-    conf.FontDataOwnedByAtlas = false;
+    const ImWchar range_user[] = {
+        0xE660, 0xE6FF,
+        0
+    };
 
-    data = assets_get("asset://data/fonts/DejaVuSans-light.ttf", &data_size);
+    add_font("asset://data/fonts/DejaVuSans.ttf", ranges, false);
+    add_font("asset://data/fonts/goxel-font.ttf", range_user, true);
+
+    #if 0
+    conf.MergeMode = true;
+    data = assets_get(
+            "asset://data/fonts/DroidSansFallbackFull.ttf", &data_size);
     assert(data);
-    io.Fonts->AddFontFromMemoryTTF((void*)data, data_size, 14 * scale,
-                                   &conf, ranges);
+    io.Fonts->AddFontFromMemoryTTF(
+            (void*)data, data_size, 14 * scale, &conf,
+            io.Fonts->GetGlyphRangesJapanese());
+    #endif
+    io.Fonts->Build();
+
     io.Fonts->GetTexDataAsRGBA32(&pixels, &width, &height);
 
     GLuint tex_id;
@@ -427,16 +479,22 @@ static int check_action_shortcut(action_t *action, void *user)
     bool check_key = true;
     bool check_char = true;
     if (!*s) return 0;
+
     if (io.KeyCtrl) {
         if (!str_startswith(s, "Ctrl")) return 0;
         s += strlen("Ctrl ");
         check_char = false;
-    } else {
-        if (str_startswith(s, "Ctrl")) return 0;
     }
+
     if (io.KeyShift) {
-        check_key = false;
+        if (!str_startswith(s, "Shift")) return 0;
+        s += strlen("Shift ");
+        check_char = false;
     }
+
+    if (str_startswith(s, "Ctrl")) return 0;
+    if (str_startswith(s, "Shift")) return 0;
+
     if (    (check_char && isCharPressed(s[0])) ||
             (check_key && ImGui::IsKeyPressed((ImGuiKey)s[0], false))) {
         action_exec(action);
@@ -469,6 +527,10 @@ static void render_popups(int index)
         flags &= ~(ImGuiWindowFlags_NoMove |
                    ImGuiWindowFlags_AlwaysAutoResize);
     }
+    if (popup->title[0] == '#') {
+        flags |= ImGuiWindowFlags_NoTitleBar;
+    }
+
     ImGui::PushStyleVar(ImGuiStyleVar_WindowPadding, ImVec2(10, 10));
     ImGui::PushStyleColor(ImGuiCol_PopupBg, COLOR(WINDOW, BACKGROUND, false));
     ImGui::PushStyleColor(ImGuiCol_TitleBgActive, COLOR(WINDOW, INNER, false));
@@ -493,14 +555,14 @@ static void render_popups(int index)
     ImGui::PopStyleVar();
 }
 
-static void render_view_cube(void)
+bool gui_view_cube(float x, float y, float w, float h)
 {
     ImGuiIO& io = ImGui::GetIO();
-
+    bool ret = false;
     ImGuizmo::SetRect(0, 0, io.DisplaySize.x, io.DisplaySize.y);
     camera_t *camera = goxel.image->active_camera;
     float view[4][4];
-    const float w = 128, h = 128;
+    float view_prev[4][4];
     const float *projection= (float*)camera->proj_mat;
     const float zup2yup[4][4] = {
         {1, 0, 0, 0},
@@ -523,10 +585,10 @@ static void render_view_cube(void)
     // XXX: ImGuizmo is using Y up.
     mat4_mul(zup2yup, camera->mat, view);
     mat4_invert(view, view);
+    mat4_copy(view, view_prev);
 
     ImGui::SetNextWindowSize(ImVec2(w, h));
-    ImGui::SetNextWindowPos(ImVec2(
-                goxel.gui.viewport[2] - w, goxel.gui.viewport[1]));
+    ImGui::SetNextWindowPos(ImVec2(x, y));
     ImGui::PushStyleColor(ImGuiCol_WindowBg, ImVec4(0, 0, 0, 0));
     ImGui::Begin("Gizmo", NULL, ImGuiWindowFlags_NoDecoration);
     ImGuizmo::SetDrawlist();
@@ -539,11 +601,14 @@ static void render_view_cube(void)
            ImGui::GetWindowPos(),
            ImVec2(w, h), 0x0);
 
-    mat4_invert(view, view);
-    mat4_mul(yup2zup, view, camera->mat);
+    ret = memcmp(view, view_prev, sizeof(view_prev)) != 0;
+    if (ret) {
+        mat4_invert(view, view);
+        mat4_mul(yup2zup, view, camera->mat);
+    }
     ImGui::End();
     ImGui::PopStyleColor();
-
+    return ret;
 }
 
 static void gui_iter(const inputs_t *inputs)
@@ -553,12 +618,13 @@ static void gui_iter(const inputs_t *inputs)
     ImGuiIO& io = ImGui::GetIO();
     ImGuiStyle& style = ImGui::GetStyle();
 
+    gui->want_capture_mouse = io.WantCaptureMouse;
     io.DisplaySize = ImVec2((float)goxel.screen_size[0],
                             (float)goxel.screen_size[1]);
 
     io.DisplayFramebufferScale = ImVec2(goxel.screen_scale,
                                         goxel.screen_scale);
-    io.DeltaTime = goxel.delta_time;
+    io.DeltaTime = fmax(goxel.delta_time, 0.01);
     io.ConfigDragClickToInputText = true;
 
     if (inputs) {
@@ -576,12 +642,12 @@ static void gui_iter(const inputs_t *inputs)
             io.KeysDown[i] = inputs->keys[i];
         io.KeyShift = inputs->keys[KEY_LEFT_SHIFT] ||
                       inputs->keys[KEY_RIGHT_SHIFT];
-        io.KeyCtrl = inputs->keys[KEY_CONTROL];
+        io.KeyCtrl = inputs->keys[KEY_LEFT_CONTROL] ||
+                     inputs->keys[KEY_RIGHT_CONTROL];
         for (i = 0; i < ARRAY_SIZE(inputs->chars); i++) {
             if (!inputs->chars[i]) break;
             io.AddInputCharacter(inputs->chars[i]);
         }
-        memset((void*)inputs->chars, 0, sizeof(inputs->chars));
     }
 
 
@@ -594,18 +660,22 @@ static void gui_iter(const inputs_t *inputs)
     style.WindowRounding = 6;
     style.ChildBorderSize = 0;
     style.SelectableTextAlign = ImVec2(0.5, 0.5);
+    style.FramePadding = ImVec2(4,
+            (GUI_ITEM_HEIGHT - ImGui::GetFontSize()) / 2);
     style.Colors[ImGuiCol_WindowBg] = COLOR(WINDOW, BACKGROUND, false);
     style.Colors[ImGuiCol_ChildBg] = COLOR(SECTION, BACKGROUND, false);
     style.Colors[ImGuiCol_Header] = ImVec4(0, 0, 0, 0);
     style.Colors[ImGuiCol_Text] = COLOR(BASE, TEXT, false);
     style.Colors[ImGuiCol_MenuBarBg] = COLOR(MENU, BACKGROUND, false);
 
+    gui->scrolling = (gui->scrolling & 1) ? 2 : 0;
+    gui->can_move_window = (gui->can_move_window & 1) ? 2 : 0;
+
     // Old code, to remove.
     ImGui::NewFrame();
     ImGuizmo::BeginFrame();
 
     gui_app();
-    render_view_cube();
     render_popups(0);
 
     // Handle the shortcuts.  XXX: this should be done with actions.
@@ -641,6 +711,8 @@ void gui_group_begin(const char *label)
     ImGui::PushID(label ?: "group");
     ImGui::BeginGroup();
     ImGui::PushStyleVar(ImGuiStyleVar_ItemSpacing, ImVec2(2, 2));
+
+    memset(&gui->groups[gui->groups_count++], 0, sizeof(gui->groups[0]));
 }
 
 void gui_group_end(void)
@@ -650,6 +722,14 @@ void gui_group_end(void)
     ImGui::Dummy(ImVec2(0, 0));
     ImGui::EndGroup();
     if (gui->is_row) ImGui::SameLine();
+
+    // Make sure gui_is_item_activated works at group level.
+    assert(gui->groups_count > 0);
+    if (gui->groups[gui->groups_count - 1].activated)
+        gui->item_activated = true;
+    if (gui->groups[gui->groups_count - 1].deactivated)
+        gui->item_deactivated = true;
+    gui->groups_count--;
 }
 
 bool gui_section_begin(const char *label, int flags)
@@ -705,32 +785,73 @@ void gui_row_end(void)
     gui->item_size = 0;
 }
 
-void gui_window_begin(const char *label, float x, float y, float w, float h,
-                      bool *moved)
+int gui_window_begin(const char *label, float x, float y, float w, float h,
+                     int flags)
 {
-    ImGuiWindowFlags flags =
-        ImGuiWindowFlags_NoTitleBar | ImGuiWindowFlags_NoResize |
+    ImGuiWindowFlags win_flags =
+        ImGuiWindowFlags_NoTitleBar | ImGuiWindowFlags_NoResize  |
         ImGuiWindowFlags_NoDecoration;
-    float max_h;
+    float max_size;
+    ImGuiStorage *storage = ImGui::GetStateStorage();
+    ImGuiID key;
+    float *last_pos;
+    int ret = 0;
+    int dir = (flags & GUI_WINDOW_HORIZONTAL) ? 0 : 1;
 
+    ImGui::PushID(label);
+    if (!gui->can_move_window)
+        win_flags |= ImGuiWindowFlags_NoMove;
+    if (gui->scrolling)
+        win_flags |= ImGuiWindowFlags_NoMouseInputs;
+    if (dir == 0)
+        win_flags |= ImGuiWindowFlags_HorizontalScrollbar;
     ImGui::SetNextWindowPos(ImVec2(x, y),
-            moved == NULL ? ImGuiCond_Always : ImGuiCond_Appearing);
+            (flags & GUI_WINDOW_MOVABLE) ?
+            ImGuiCond_Appearing : ImGuiCond_Always);
     ImGui::SetNextWindowSize(ImVec2(w, h));
-    if (h == 0) {
-        max_h = ImGui::GetMainViewport()->Size.y - y;
-        ImGui::SetNextWindowSizeConstraints(
-                ImVec2(0, 0), ImVec2(FLT_MAX, max_h));
-    }
-    ImGui::Begin(label, NULL, flags);
 
-    if (moved != NULL) {
-        *moved = ImGui::GetWindowPos() != ImVec2(x, y);
+    key = ImGui::GetID("last_pos");
+    last_pos = storage->GetFloatRef(key, dir == 0 ? x : y);
+
+    if ((w == 0) && (dir == 0)) {
+        max_size = ImGui::GetMainViewport()->Size.x - *last_pos;
+        ImGui::SetNextWindowSizeConstraints(
+                ImVec2(0, 0), ImVec2(max_size, FLT_MAX));
     }
+    if ((h == 0) && (dir == 1)) {
+        max_size = ImGui::GetMainViewport()->Size.y - *last_pos;
+        ImGui::SetNextWindowSizeConstraints(
+                ImVec2(0, 0), ImVec2(FLT_MAX, max_size));
+    }
+
+    ImGui::Begin(label, NULL, win_flags);
+
+    if (flags & GUI_WINDOW_MOVABLE) {
+         if (ImGui::GetWindowPos() != ImVec2(x, y))
+             ret |= GUI_WINDOW_MOVED;
+        *last_pos = dir == 0 ? ImGui::GetWindowPos().x :
+                               ImGui::GetWindowPos().y;
+    }
+
+    gui->win_dir = dir;
+    ImGui::BeginGroup();
+    return ret;
 }
 
-void gui_window_end(void)
+gui_window_ret_t gui_window_end(void)
 {
+    gui_window_ret_t ret = {};
+    ImGui::EndGroup();
+    if (!GUI_HAS_SCROLLBARS && !gui->can_move_window) {
+        if (gui_pan_scroll_behavior(gui->win_dir))
+            gui->scrolling |= 1;
+    }
+    ret.h = ImGui::GetWindowHeight();
+    ret.w = ImGui::GetWindowWidth();
     ImGui::End();
+    ImGui::PopID();
+
+    return ret;
 }
 
 bool gui_input_int(const char *label, int *v, int minv, int maxv)
@@ -739,10 +860,6 @@ bool gui_input_int(const char *label, int *v, int minv, int maxv)
     float maxvf = maxv;
     bool ret;
     float vf = *v;
-    if (minv == 0 && maxv == 0) {
-        minvf = -FLT_MAX;
-        maxvf = +FLT_MAX;
-    }
     ret = gui_input_float(label, &vf, 1, minvf, maxvf, "%.0f");
     if (ret) *v = vf;
     return ret;
@@ -811,32 +928,64 @@ static bool slider_float(float *v, float minv, float maxv, const char *format)
     return ret;
 }
 
+// To be called after imgui inputs function, to udpate our internal state.
+static void update_activation_state(void)
+{
+    int i;
+    if (ImGui::IsItemActivated()) {
+        gui->item_activated = true;
+        for (i = 0; i < gui->groups_count; i++) {
+            gui->groups[i].activated = true;
+        }
+    }
+    if (ImGui::IsItemDeactivated()) {
+        gui->item_deactivated = true;
+        for (i = 0; i < gui->groups_count; i++) {
+            gui->groups[i].deactivated = true;
+        }
+    }
+}
+
 bool gui_input_float(const char *label, float *v, float step,
                      float minv, float maxv, const char *format)
 {
     bool ret = false;
-    float button_width = 20; // Compute exactly.
     const char *left_utf = "◀";
     const char *right_utf = "▶";
     float v_speed = step / 10;
     bool unbounded;
     bool show_arrows = false;
     bool is_active = false;
-    ImGuiID key;
+    ImGuiID key = 0;
     ImGuiStorage *storage = ImGui::GetStateStorage();
+    const ImGuiStyle& style = ImGui::GetStyle();
+    const ImVec2 button_size = ImVec2(
+            GUI_ITEM_HEIGHT,
+            ImGui::GetFontSize() + style.FramePadding.y * 2.0f);
+    int v_int;
+    char v_label[128];
 
-    if (minv == 0.f && maxv == 0.f) {
-        minv = -FLT_MAX;
-        maxv = +FLT_MAX;
+    gui->item_activated = false;
+    gui->item_deactivated = false;
+
+    // To remove: use 0, 0 instead.
+    if (minv == -FLT_MAX && maxv == +FLT_MAX) {
+        assert(false);
+        minv = 0;
+        maxv = 0;
     }
+
     if (step == 0.f) step = 0.1f;
     if (!format) format = "%.1f";
 
-    unbounded = (minv == -FLT_MAX || maxv == +FLT_MAX);
+    // Because imgui doesn't round to the step, we use DragInt with a
+    // custom label to make it work properly.
+    v_int = round(*v / step);
+    snprintf(v_label, sizeof(v_label), format, v_int * step);
+    unbounded = (minv == 0 && maxv == 0);
 
     ImGui::PushID(label);
 
-
     ImGui::PushStyleColor(ImGuiCol_FrameBg, COLOR(NUMBER_INPUT, INNER, false));
     ImGui::PushStyleColor(ImGuiCol_FrameBgHovered,
                     color_lighten(COLOR(NUMBER_INPUT, INNER, false)));
@@ -861,25 +1010,35 @@ bool gui_input_float(const char *label, float *v, float step,
     }
 
     if (show_arrows) {
-        if (ImGui::Button(left_utf)) {
+        if (ImGui::Button(left_utf, button_size)) {
             (*v) -= step;
             ret = true;
         }
+        update_activation_state();
+
         ImGui::SameLine();
         ImGui::PushItemWidth(
-                ImGui::GetContentRegionAvail().x - button_width);
-        ret = ImGui::DragFloat("", v, v_speed, minv, maxv, format) || ret;
+                ImGui::GetContentRegionAvail().x - button_size.x - 4);
+        if (ImGui::DragInt("", &v_int, v_speed, minv, maxv, v_label)) {
+            *v  = v_int * step;
+            ret = true;
+        }
+        update_activation_state();
         is_active = ImGui::IsItemActive();
         ImGui::PopItemWidth();
         ImGui::SameLine();
-        if (ImGui::Button(right_utf)) {
+        if (ImGui::Button(right_utf, button_size)) {
             (*v) += step;
             ret = true;
         }
+        update_activation_state();
     } else {
         ImGui::SetNextItemWidth(-1);
         if (unbounded) {
-            ret = ImGui::DragFloat("", v, step, minv, maxv, format);
+            if (ImGui::DragInt("", &v_int, step, minv, maxv, v_label)) {
+                *v  = v_int * step;
+                ret = true;
+            }
         } else {
             ret = slider_float(v, minv, maxv, format);
         }
@@ -897,7 +1056,9 @@ bool gui_input_float(const char *label, float *v, float step,
     ImGui::PopID();
 
     if (ret) {
-        *v = clamp(*v, minv, maxv);
+        if (minv != 0 || maxv != 0) {
+            *v = clamp(*v, minv, maxv);
+        }
         on_click();
     }
     return ret;
@@ -915,12 +1076,12 @@ bool gui_bbox(float box[4][4])
     y = round(box[3][1] - box[1][1]);
     z = round(box[3][2] - box[2][2]);
 
-    gui_group_begin("Origin");
+    gui_group_begin(_("Origin"));
     ret |= gui_input_int("x", &x, 0, 0);
     ret |= gui_input_int("y", &y, 0, 0);
     ret |= gui_input_int("z", &z, 0, 0);
     gui_group_end();
-    gui_group_begin("Size");
+    gui_group_begin(_("Size"));
     ret |= gui_input_int("w", &w, 0, 0);
     ret |= gui_input_int("h", &h, 0, 0);
     ret |= gui_input_int("d", &d, 0, 0);
@@ -963,8 +1124,8 @@ bool gui_action_button(int id, const char *label, float size)
     assert(action);
     ImGui::PushID(action->id);
     ret = gui_button(label, size, action->icon);
-    if (ImGui::IsItemHovered())
-        goxel_set_help_text(action_get(id, true)->help);
+    if (ImGui::IsItemHovered() && action->help)
+        goxel_add_hint(0, NULL, tr(action->help));
     if (ret) {
         action_exec(action_get(id, true));
     }
@@ -987,8 +1148,8 @@ static bool _selectable(const char *label, bool *v, const char *tooltip,
 
     v = v ? v : &default_v;
     size = (icon != -1) ?
-        ImVec2(ICON_HEIGHT, ICON_HEIGHT) :
-        ImVec2(w, ITEM_HEIGHT);
+        ImVec2(GUI_ICON_HEIGHT, GUI_ICON_HEIGHT) :
+        ImVec2(w, GUI_ITEM_HEIGHT);
 
     if (!tooltip && icon != -1) {
         tooltip = label;
@@ -1029,7 +1190,7 @@ static bool _selectable(const char *label, bool *v, const char *tooltip,
     if (ret) *v = !*v;
     if (tooltip && ImGui::IsItemHovered()) {
         gui_tooltip(tooltip);
-        goxel_set_help_text(tooltip);
+        goxel_add_hint(0, NULL, tooltip);
     }
     ImGui::PopID();
     if (gui->is_row) ImGui::SameLine();
@@ -1127,7 +1288,7 @@ static bool color_picker(const char *label, uint8_t color[4])
 bool gui_color(const char *label, uint8_t color[4])
 {
     bool ret = false;
-    ImVec2 size(ICON_HEIGHT, ICON_HEIGHT);
+    ImVec2 size(GUI_ICON_HEIGHT, GUI_ICON_HEIGHT);
 
     ImGui::PushID(label);
     if (ImGui::ColorButton(label, color, 0, size)) {
@@ -1142,6 +1303,7 @@ bool gui_color(const char *label, uint8_t color[4])
     }
 
     ImGui::PopID();
+    if (gui->is_row) ImGui::SameLine();
     return ret;
 }
 
@@ -1212,16 +1374,17 @@ bool gui_button(const char *label, float size, int icon)
     ImVec2 center;
     int w, isize;
 
-    button_size = ImVec2(size * ImGui::GetContentRegionAvail().x, ITEM_HEIGHT);
+    button_size = ImVec2(size * ImGui::GetContentRegionAvail().x,
+                         GUI_ITEM_HEIGHT);
     if (size == -1) button_size.x = ImGui::GetContentRegionAvail().x;
     if (size == 0 && (label == NULL || label[0] == '#')) {
-        button_size.x = ICON_HEIGHT;
-        button_size.y = ICON_HEIGHT;
+        button_size.x = GUI_ICON_HEIGHT;
+        button_size.y = GUI_ICON_HEIGHT;
     }
     if (size == 0 && label && label[0] != '#') {
         w = ImGui::CalcTextSize(label, NULL, true).x + style.FramePadding.x * 2;
-        if (w < ITEM_HEIGHT)
-            button_size.x = ITEM_HEIGHT;
+        if (w < GUI_ITEM_HEIGHT)
+            button_size.x = GUI_ITEM_HEIGHT;
     }
 
     if (gui->item_size) button_size.x = gui->item_size;
@@ -1231,6 +1394,7 @@ bool gui_button(const char *label, float size, int icon)
             (label && (label[0] != '#')) ?
             COLOR(BUTTON, INNER, false) : COLOR(ICON, INNER, false));
     ret = ImGui::Button(label ?: "", button_size);
+    update_activation_state();
     ImGui::PopStyleColor();
     if (icon) {
         center = ImGui::GetItemRectMin() +
@@ -1243,7 +1407,12 @@ bool gui_button(const char *label, float size, int icon)
                             center + ImVec2(isize, isize),
                             uv0, uv1, get_icon_color(icon, 0));
     }
-    if (ret) on_click();
+    if (ret) {
+        on_click();
+        if (gui->is_context_menu)  {
+            ImGui::CloseCurrentPopup();
+        }
+    }
     if (gui->is_row) ImGui::SameLine();
     return ret;
 }
@@ -1253,7 +1422,7 @@ bool gui_button_right(const char *label, int icon)
     const ImGuiStyle& style = ImGui::GetStyle();
     float text_size = ImGui::CalcTextSize(label).x;
     float w = text_size + 2 * style.FramePadding.x;
-    w = max(w, ITEM_HEIGHT);
+    w = max(w, GUI_ITEM_HEIGHT);
     w += style.FramePadding.x;
     ImGui::SameLine();
     ImGui::Dummy(ImVec2(ImGui::GetContentRegionAvail().x - w, 0));
@@ -1344,10 +1513,12 @@ void gui_enabled_begin(bool enabled)
     ImVec4 color = style.Colors[ImGuiCol_Text];
     if (!enabled) color.w /= 2;
     ImGui::PushStyleColor(ImGuiCol_Text, color);
+    ImGui::BeginDisabled(!enabled);
 }
 
 void gui_enabled_end(void)
 {
+    ImGui::EndDisabled();
     ImGui::PopStyleColor();
 }
 
@@ -1456,8 +1627,9 @@ void gui_on_popup_closed(void (*func)(int))
 
 void gui_popup_bottom_begin(void)
 {
-    float w = ImGui::GetContentRegionAvail().y -
-              ImGui::GetFrameHeightWithSpacing();
+    const ImGuiStyle& style = ImGui::GetStyle();
+    float bottom_y = GUI_ITEM_HEIGHT + style.FramePadding.y * 2;
+    float w = ImGui::GetContentRegionAvail().y - bottom_y;
     ImGui::Dummy(ImVec2(0, w));
     gui_row_begin(0);
 }
@@ -1509,25 +1681,29 @@ void gui_request_panel_width(float width)
     goxel.gui.panel_width = width;
 }
 
-bool gui_layer_item(int i, int icon, bool *visible, bool *edit,
+bool gui_layer_item(int idx, int icons_count, const int *icons,
+                    bool *visible, bool *selected,
                     char *name, int len)
 {
     bool ret = false;
-    bool edit_ = *edit;
+    bool selected_ = *selected;
     static char *edit_name = NULL;
     static bool start_edit;
     float font_size = ImGui::GetFontSize();
+    int icon;
+    int i;
     ImVec2 center;
     ImVec2 uv0, uv1;
+    ImVec2 padding;
     ImDrawList* draw_list = ImGui::GetWindowDrawList();
     ImGuiStyle& style = ImGui::GetStyle();
 
-    ImGui::PushID(i);
-    ImGui::PushStyleColor(ImGuiCol_Button, COLOR(WIDGET, INNER, *edit));
+    ImGui::PushID(idx);
+    ImGui::PushStyleColor(ImGuiCol_Button, COLOR(WIDGET, INNER, *selected));
     ImGui::PushStyleColor(ImGuiCol_ButtonHovered,
-            color_lighten(COLOR(WIDGET, INNER, *edit)));
+            color_lighten(COLOR(WIDGET, INNER, *selected)));
     if (visible) {
-        if (gui_selectable_icon("##visible", &edit_,
+        if (gui_selectable_icon("##visible", &selected_,
                 *visible ? ICON_VISIBILITY : ICON_VISIBILITY_OFF)) {
             *visible = !*visible;
             ret = true;
@@ -1537,18 +1713,19 @@ bool gui_layer_item(int i, int icon, bool *visible, bool *edit,
 
     if (edit_name != name) {
         ImGui::PushStyleVar(ImGuiStyleVar_ButtonTextAlign, ImVec2(0, 0.5));
-        if (icon != -1) {
-            ImGui::PushStyleVar(ImGuiStyleVar_FramePadding,
-                    ImVec2(ICON_HEIGHT / 1.5, 0));
-        }
-        if (ImGui::Button(name, ImVec2(-1, ICON_HEIGHT))) {
-            *edit = true;
+        padding = style.FramePadding;
+        padding.x += GUI_ICON_HEIGHT * 0.75 * icons_count;
+        ImGui::PushStyleVar(ImGuiStyleVar_FramePadding, padding);
+        if (ImGui::Button(name, ImVec2(-1, GUI_ICON_HEIGHT))) {
+            *selected = true;
             ret = true;
         }
-        if (icon != -1) ImGui::PopStyleVar();
-        if (icon > 0) {
+        ImGui::PopStyleVar();
+
+        for (i = 0; i < icons_count; i++) {
+            icon = icons[i];
             center = ImGui::GetItemRectMin() +
-                ImVec2(ICON_HEIGHT / 2 / 1.5, ICON_HEIGHT / 2);
+                ImVec2(GUI_ICON_HEIGHT * 0.75 * (i + 0.5), GUI_ICON_HEIGHT / 2);
             uv0 = ImVec2(((icon - 1) % 8) / 8.0, ((icon - 1) / 8) / 8.0);
             uv1 = ImVec2(uv0.x + 1. / 8, uv0.y + 1. / 8);
             draw_list->AddImage(
@@ -1566,7 +1743,7 @@ bool gui_layer_item(int i, int icon, bool *visible, bool *edit,
         if (start_edit) ImGui::SetKeyboardFocusHere();
         ImGui::PushStyleVar(ImGuiStyleVar_FramePadding,
                             ImVec2(style.FramePadding.x,
-                            (ICON_HEIGHT - font_size) / 2));
+                            (GUI_ICON_HEIGHT - font_size) / 2));
         ImGui::InputText("##name_edit", name, len,
                          ImGuiInputTextFlags_AutoSelectAll);
         if (!start_edit && !ImGui::IsItemActive()) edit_name = NULL;
@@ -1634,7 +1811,7 @@ bool gui_menu_item(int action, const char *label, bool enabled)
 
 void gui_tooltip(const char *str)
 {
-    if (gui->is_scrolling) return;
+    if (gui->scrolling) return;
     ImGui::PushStyleColor(ImGuiCol_PopupBg, COLOR(TOOLTIP, BACKGROUND, 0));
     ImGui::SetTooltip("%s", str);
     ImGui::PopStyleColor();
@@ -1654,12 +1831,12 @@ static bool panel_header_close_button(void)
     ImDrawList* draw_list = ImGui::GetWindowDrawList();
     bool ret;
 
-    w = ITEM_HEIGHT + style.FramePadding.x;
+    w = GUI_ITEM_HEIGHT + style.FramePadding.x;
     ImGui::SameLine();
     ImGui::Dummy(ImVec2(ImGui::GetContentRegionAvail().x - w, 0));
     ImGui::SameLine();
     ImGui::PushStyleColor(ImGuiCol_Button, ImVec4(0, 0, 0, 0));
-    ret = ImGui::Button("", ImVec2(ITEM_HEIGHT, ITEM_HEIGHT));
+    ret = ImGui::Button("", ImVec2(GUI_ITEM_HEIGHT, GUI_ITEM_HEIGHT));
     ImGui::PopStyleColor();
 
     center = ImGui::GetItemRectMin() +
@@ -1678,15 +1855,19 @@ bool gui_panel_header(const char *label)
 {
     bool ret;
     float label_w = ImGui::CalcTextSize(label).x;
-    float w = ImGui::GetContentRegionAvail().x - ITEM_HEIGHT;
+    float w = ImGui::GetContentRegionAvail().x - GUI_ITEM_HEIGHT;
 
     ImGui::PushID("panel_header");
+    ImGui::BeginGroup();
     ImGui::Dummy(ImVec2((w - label_w) / 2, 0));
     ImGui::SameLine();
     ImGui::AlignTextToFramePadding();
     gui_text(label);
     ret = panel_header_close_button();
+    ImGui::EndGroup();
     ImGui::PopID();
+    if (ImGui::IsItemHovered())
+        gui->can_move_window |= 1;
     return ret;
 }
 
@@ -1726,10 +1907,10 @@ bool gui_icons_grid(int nb, const gui_icon_info_t *icons, int *current)
         }
         v = (i == *current);
         if (!is_colors_grid) {
-            size = ICON_HEIGHT;
+            size = GUI_ICON_HEIGHT;
             clicked = gui_selectable_icon(label, &v, icon->icon);
         } else { // Color icon.
-            size = ITEM_HEIGHT;
+            size = GUI_ITEM_HEIGHT;
             ImGui::PushStyleColor(ImGuiCol_Button, icon->color);
             ImGui::PushStyleColor(ImGuiCol_ButtonHovered, icon->color);
             clicked = ImGui::Button("", ImVec2(size, size));
@@ -1762,7 +1943,59 @@ bool gui_icons_grid(int nb, const gui_icon_info_t *icons, int *current)
 
 bool gui_want_capture_mouse(void)
 {
-    gui_init();
-    ImGuiIO& io = ImGui::GetIO();
-    return io.WantCaptureMouse;
+    return gui && gui->want_capture_mouse;
+}
+
+bool gui_context_menu_begin(const char *label)
+{
+    bool ret;
+    ImGui::PushStyleVar(ImGuiStyleVar_WindowPadding, ImVec2(10, 10));
+    ImGui::PushStyleColor(ImGuiCol_PopupBg, COLOR(WINDOW, BACKGROUND, false));
+    ImGui::PushStyleColor(ImGuiCol_TitleBgActive, COLOR(WINDOW, INNER, false));
+
+    ret = ImGui::BeginPopupContextItem(label);
+    if (ret) {
+        gui->is_context_menu = true;
+        gui->context_menu_row = gui->is_row;
+        gui->is_row = 0;
+        gui_group_begin(NULL);
+    }
+    ImGui::PopStyleColor(2);
+    ImGui::PopStyleVar();
+    return ret;
+}
+
+void gui_context_menu_end(void)
+{
+    gui_group_end();
+    ImGui::EndPopup();
+    gui->is_context_menu = false;
+    gui->is_row = gui->context_menu_row;
+}
+
+void gui_context_menu_button(const char *label, int icon)
+{
+    if (gui_button(label, 0, icon)) {
+        ImGui::OpenPopup(label);
+    }
+}
+
+bool gui_is_item_activated(void)
+{
+    return gui->item_activated;
+}
+
+bool gui_is_item_deactivated(void)
+{
+    return gui->item_deactivated;
+}
+
+float gui_get_current_pos_x(void)
+{
+    return ImGui::GetCursorPosX();
+}
+
+void gui_set_current_pos_x(float x)
+{
+    ImGui::SetCursorPosX(x);
 }
