@@ -27,21 +27,6 @@ typedef struct {
 int tool_color_picker_iter(tool_t *tool, const painter_t *painter,
                            const float viewport[4])
 {
-    uint8_t color[4];
-    const volume_t *volume = goxel_get_layers_volume(goxel.image);
-    cursor_t *curs = &goxel.cursor;
-    int pi[3] = {floor(curs->pos[0]),
-                 floor(curs->pos[1]),
-                 floor(curs->pos[2])};
-    curs->snap_mask = SNAP_VOLUME;
-    curs->snap_offset = -0.5;
-
-    goxel_set_help_text("Click on a voxel to pick the color");
-    if (!curs->snaped) return 0;
-    volume_get_at(volume, NULL, pi, color);
-    color[3] = 255;
-    goxel_set_help_text("%d %d %d", color[0], color[1], color[2]);
-    if (curs->flags & CURSOR_PRESSED) vec4_copy(color, goxel.painter.color);
     return 0;
 }
 
@@ -52,8 +37,9 @@ static int gui(tool_t *tool)
 }
 
 TOOL_REGISTER(TOOL_PICK_COLOR, pick_color, tool_pick_color_t,
-             .name = "Color Picker",
+             .name = N_("Color Picker"),
              .iter_fn = tool_color_picker_iter,
              .gui_fn = gui,
              .default_shortcut = "C",
+             .flags = TOOL_ALLOW_PICK_COLOR,
 )
